#
# This file is part of python sample package for the BROADWAY2 package.
#
# MIT License

# Copyright (c) 2013-2019 FibreCode GmbH <info@fibrecode.com>

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.


import sys, os
import binascii
import platform
import time, datetime
import threading, queue
import binascii

from broadway2 import Broadway, RawReceiver, RawRxHeaderTypes, EthFrame, RawRxFrame
from raw_checker import RawChecker


def main(args):
    
    # create adapter singleton
    base = Broadway()

    
    print('='*25)

    aList = base.get_raw_adapters()

    if len(aList) < 1:
        print('no Raw-Adapter found')
        sys.exit(-1)


    if len(aList) > 1:
        cnt = 1
        for a in aList:
            print('[ {0:2} ]: {1} / {2}'.format(cnt, a[1], a[0]))
            cnt += 1
        try:
            sel = int(input('Enter Number to use:'))
        except:
            sel = 0
        
    else:
        sel = 0


    inst, friendlyName = aList[sel-1]    # get adapter selected by instance     
         
    if sel >= 0:
        devRx = base.open_raw_rx(inst, RawRxHeaderTypes.HeaderType2)
        print('Raw Rx Device opened: {0} / {1}'.format(friendlyName, inst))

        rawChecker = RawChecker(1)

        rawReceiver = RawReceiver(devRx)

        rawReceiver.enable()

        print('Receiver running... Ctrl-C to exit')
        timePrint = time.monotonic() - 6    # force to print direct at start
        packetsPerSec = 0.0
        bytesPerSec = 0.0
        lastTime = 0
        try:
            while 1:
                # read_block with max. 500 frames and timeout after 500 ms
                result = rawReceiver.read_block(100, 100)
                
                    
                rawChecker.check_sequence(result)

                tm = time.monotonic()
                if tm - timePrint > 5:
                    if lastTime > 0:
                        packetsPerSec = (rawChecker.packetsReceived - lastPacketsReceived) / (tm-lastTime)
                        bytesPerSec = (rawChecker.bytesReceived - lastBytesReceived) / (tm-lastTime)
                    print('Time: {0}'.format(tm))
                    print('Errors: Seq:{0} / Time:{1}'.format(rawChecker.seqErrors, rawChecker.tmErrors))
                    print('DataErr: {0}'.format(rawChecker.dataErrors))
                    print('Statistics  : LLC:{0} Packets: {1} / Bytes: {2}'.format(rawChecker.llcFramesCnt ,rawChecker.packetsReceived, rawChecker.bytesReceived))
                    print(' Packets/Sec: {0} / kBytes/sec: {1:.4f}'.format(packetsPerSec, bytesPerSec / 1000.0))
                    print()
                    print('LastFrame received: {0} / {1}'.format(rawChecker.lastRawFrameReceived, rawChecker.lastFrameId))
                    print('---')
                    lastPacketsReceived = rawChecker.packetsReceived
                    lastBytesReceived = rawChecker.bytesReceived
                    lastTime = tm
                    timePrint = tm           
                    
        except KeyboardInterrupt:

            rawReceiver.disable()

            base.close(devRx)


if __name__ == "__main__":
    main(sys.argv)

